######################################################
## YUIMA tutorial R-example codes (2018/12/2)
######################################################

library(yuima)

######################################################
## Example 1: Compound Poisson jumps
######################################################
set.seed(123)

mod.cpp <- setPoisson(intensity = "lambda", df = list("dnorm(z,mu,sigma)"))

T <- 10   # 最終観測時点
samp <- setSampling(Initial = 0, Terminal = T, n = 100*T)   # 各区間[i-1,i]で100買個データ生成
cpp <- setYuima(model = mod.cpp, sampling = samp)           # モデルを格納
param <- list(lambda = 10, mu = 0, sigma = 2)               # モデルパラメータを先にまとめて設定
str(cpp)    # 中身をみる

X.cpp <- simulate(cpp, xinit = 0, true.parameter = param)   # CPPのパス生成
plot(X.cpp, col = "blue", main="Compound Poisson process")  # プロット


######################################################
## Example 2: IG-Levy process (subordinator)
######################################################
set.seed(123)
T <- 1
n <- 1000
jump <- "1"

mod.ig <- setModel(drift = "0", diffusion="0", jump.coeff = jump, 
                   solve.variable = "x",state.variable = "x", 
                   measure.type = "code", measure = list(df="rIG(z,3,3)"))

samp <- setSampling(Terminal=T, n=n)
ig <- setYuima(model=mod.ig, sampling=samp)

str(ig)

X.ig <- simulate(ig, xinit=0)
plot(X.ig, col="blue", main="IG-Levy process (Subordinator)")


######################################################
## Example 3: NIG-Levy process
######################################################
set.seed(123)
T <- 1
n <- 1000
jump <- "1"

mod.nig <- setModel(drift = "0", diffusion="0", jump.coeff = jump, 
                    solve.variable = "x",state.variable = "x", 
                    measure.type = "code", measure = list(df="rNIG(z,3,0,3,0)"))

samp <- setSampling(Terminal=T, n=n)
nig <- setYuima(model=mod.nig, sampling=samp)

str(nig)

X.nig <- simulate(nig, xinit=0)
plot(X.nig, col="blue", main="NIG-Levy process")


## Histgram checking: cf. "help(rng)" in YUIMA package
n <- 100000
X.nig <- rNIG(n,5,-0.5,5,0)           # NIG(5,-0.5,5,0)-乱数生成
pdf <- function(x) dNIG(x,5,-0.5,5,0) # NIG(5,-0.5,5,0)-pdfを格納
hist(X.nig, prob=T, breaks=50, col="lightblue", xlim=c(-5,4),ylim=c(0,0.45),xlab="",ylab="Empirical density",main="NIG-rng histgram checking")
curve(pdf, add = TRUE, col="red", lwd = 2)  # ターゲット密度関数を上書き

######################################################
## Example 4: Diffusion with compound Poisson jumps
######################################################
set.seed(123)
T <- 1
n <- 1000
jump <- "-1"

mod.dcppj <- setModel(drift=c("sin(x)-theta*x"), diffusion="sigma", jump.coeff=jump,
                      measure=list(intensity="lambda",df=list("dgamma(z,3,3)")),
                      measure.type="CP",solve.variable="x")
samp <- setSampling(Terminal=T, n=n)
dcppj <- setYuima(model=mod.dcppj, sampling=samp)

param <- list(lambda = 10, theta = 1, sigma = 2)

str(dcppj)

X.dcppj <- simulate(dcppj, xinit=0, true.parameter=param)
plot(X.dcppj, col="blue", main="Diffusion with compound Poisson jumps")


######################################################
## Example 5: Geometric NIG-Levy process
######################################################
set.seed(123)
T <- 1
n <- 1000
drift <- "0"  ## <-- アブストに不備あり（"x"でなく0"が正しい）
diff <- "0"   ## <-- アブストに不備あり（"x"でなく0"が正しい）
jump <- "x"

mod.gnig <- setModel(drift = drift, diffusion=diff, jump.coeff = jump, 
                     solve.variable = "x",state.variable = "x", 
                     measure.type = "code", measure = list(df="rNIG(z,5,0,5,0)"))

samp <- setSampling(Terminal=T, n=n)
gnig <- setYuima(model=mod.gnig, sampling=samp)

str(gnig)

X.gnig <- simulate(gnig, xinit=1)
plot(X.gnig, col="blue", main="Geometric NIG-Levy process")


######################################################
## Example 6: Non-linear stable-Levy SDE
######################################################
set.seed(123)
T <- 1
n <- 1000

drift <- "-x/sqrt(1+x^2)"
jump <- "1"

mod.st <- setModel(drift = drift, diffusion="0", jump.coeff = jump, 
                   solve.variable = "x",state.variable = "x", 
                   measure.type = "code", measure = list(df="rstable(z,1.3,0,1,0)"))

samp <- setSampling(Terminal=T, n=n)
st <- setYuima(model=mod.st, sampling=samp)

str(st)

X.st <- simulate(st, xinit=0)
plot(X.st, col="blue", main="Stable-Levy SDE")


######################################################
## Example 7: 2-dim. NIG SDE
######################################################
set.seed(123)
T <- 1
n <- 1000

x0 <- c(0,0)
beta <- c(0.5,-0.3)
mu <- c(0,0)
delta0 <- 1   # <-- この値を大きくすると拡散過程にみえてくる
alpha <- 5
Lambda <- matrix(c(1,0,0,1),2,2)

jump <- matrix(c("1/sqrt(x1^2+1)","0","-0.5","1"),2,2)

mod.nigsde2 <- setModel(drift=c("-2*x1","0.3*x1-1/sqrt(1+x2^2)"), xinit=x0, 
                        solve.variable=c("x1","x2"), jump.coeff=jump, measure.type="code",
                        measure=list(df="rNIG(z,alpha,beta,delta0,mu,Lambda)"))

samp <- setSampling(Terminal=T, n=n)
nigsde2 <- setYuima(model=mod.nigsde2, sampling=samp)

str(nigsde2)

X.nigsde2 <- simulate(nigsde2, true.par=list(alpha=alpha, beta=beta, delta0=delta0, mu=mu, Lambda=Lambda))
plot(X.nigsde2, col="blue", main="2-dimensional NIG-SDE")

